#!/usr/bin/perl -w
#
# dpme.pm = disk partition map entry module
# by Brian Bresen <bbresen@linuxppc.org>
# under the GPL.
#

package dpme;

use strict;
use Fcntl;
my $NULL = 0;

sub new {
	my $class = shift @_;
	
	my $self = {};

	bless ($self, $class);
	$self->prev_by_disk( $NULL );
	$self->next_by_disk( $NULL );
	$self->prev_by_base( $NULL );
	$self->next_by_base( $NULL );

	return $self;
}

sub next_by_disk {
	my $node = shift @_;
	return @_ ? ($node->{next_by_disk} = shift) : $node->{next_by_disk};
}

sub prev_by_disk {
	my $node = shift;
	return @_ ? ($node->{prev_by_disk} = shift) : $node->{prev_by_disk};
}

sub next_by_base {
	my $node = shift;
	return @_ ? ($node->{next_by_base} = shift) : $node->{next_by_base};
}

sub prev_by_base {
	my $node = shift;
	return @_ ? ($node->{prev_by_base} = shift) : $node->{prev_by_base};
}

sub destroy {				#This may get crazy since there are two lists
	my $node = shift;
	while ($node != $NULL) {
		my $next = $node->next_by_disk;
		$node->prev_by_disk(undef);
		$node->next_by_disk(undef);
		$node->prev_by_base(undef);
		$node->next_by_base(undef);
		$node = $next;
	}
}

sub keys {
	#dpmeAddrs is strange
	return ('dpmeSig', 'dpmeReserved1', 'dpmeMapEntry', 'dpmePblockStart', 
		'dpmePblocks', 'dpmeName', 'dpmeType', 'dpmeLblockStart',
		'dpmeLblocks', 'dpmeFlags', 'dpmeBootBlock', 'dpmeBootBytes',
		'dpmeChecksum', 'dpmeProcId', 'dpmeBootArgs', 'dpmeReserved3');
}


sub _link_disk_to {
	my $node = shift;
	my $next = shift;

	$node->next_by_disk($next);
	return $next->prev_by_disk($node);
}

sub _link_base_to {
	my $node = shift;
	my $next = shift;

	$node->next_by_base($next);
	return $next->prev_by_base($node);
}

sub appendDiskOrder {
	my $node = shift;
	my $add = shift;

	$add->prev_by_disk($node);
	$add->next_by_disk($node->next_by_disk);
	$node->next_by_disk($add);

	return $node;
}

sub appendBaseOrder {
	my $node = shift;
	my $add = shift;

	$add->prev_by_base($node);
	$add->next_by_base($node->next_by_base);
	$node->next_by_base($add);

	return $node;
}

sub prependBaseOrder {
	my $node = shift;
	my $add = shift;

	$add->next_by_base($node);
	$add->prev_by_base($node->prev_by_base);
	$node->prev_by_base($add);

	return $node;
}

sub remove {
	my $first = shift;
	my $last = shift || $first;

	$first->prev_by_disk->_link_disk_to($last->next_by_disk);
	$last->_link_disk_to($first);

	$first->prev_by_base->_link_base_to($last->next_by_base);
	$last->_link_base_to($first);

	return $first;
}

sub dpmeSig {
	my $self = shift;

	return @_ ? ($self->{dpmeSig} = shift) : $self->{dpmeSig};
}

sub dpmeReserved1 {
	my $self = shift;

	return @_ ? ($self->{dpmeReserved1} = shift) : $self->{dpmeReserved1};
}

sub dpmeMapEntry {
	my $self = shift;

	return @_ ? ($self->{dpmeMapEntry} = shift) : $self->{dpmeMapEntry};
}

sub dpmePblockStart {
	my $self = shift;

	return @_ ? ($self->{dpmePblockStart} = shift) : $self->{dpmePblockStart};
}

sub dpmePblocks {
	my $self = shift;

	return @_ ? ($self->{dpmePblocks} = shift) : $self->{dpmePblocks};
}

sub dpmeName {
	my $self = shift;

	return @_ ? ($self->{dpmeName} = shift) : $self->{dpmeName};
}

sub dpmeType {
	my $self = shift;

	return @_ ? ($self->{dpmeType} = shift) : $self->{dpmeType};
}

sub dpmeLblockStart {
	my $self = shift;

	return @_ ? ($self->{dpmeLblockStart} = shift) : $self->{dpmeLblockStart};
}

sub dpmeLblocks {
	my $self = shift;

	return @_ ? ($self->{dpmeLblocks} = shift) : $self->{dpmeLblocks};
}

sub dpmeFlags {
	my $self = shift;

	return @_ ? ($self->{dpmeFlags} = shift) : $self->{dpmeFlags};
}

sub dpmeBootBlock {
	my $self = shift;

	return @_ ? ($self->{dpmeBootBlock} = shift) : $self->{dpmeBootBlock};
}

sub dpmeBootBytes {
	my $self = shift;

	return @_ ? ($self->{dpmeBootBytes} = shift) : $self->{dpmeBootBytes};
}

sub dpmeAddrs {
	my $self = shift @_;
	my ($num) = shift @_;

	return @_ ? ($self->{dpmeAddrs}[$num] = shift) : $self->{dpmeAddrs}[$num];
}

sub dpmeChecksum {
	my $self = shift;

	return @_ ? ($self->{dpmeChecksum} = shift) : $self->{dpmeChecksum};
}

sub dpmeProcId {
	my $self = shift; 

	return @_ ? ($self->{dpmeProcId} = shift) : $self->{dpmeProcId};
}

sub dpmeBootArgs {
	my $self = shift;

	return @_ ? ($self->{dpmeBootArgs} = shift) : $self->{dpmeBootArgs};
}

sub dpmeReserved3 {
	my $self = shift;

	return @_ ? ($self->{dpmeReserved3} = shift) : $self->{dpmeReserved3};
}

1;
